/******************************************************************************
 * COPYRIGHT:                                                                 * 
 *                                                                            *
 * This file is the property of the author and Communication Systems Group,   *
 * Technische Universitt Berlin. All rights reserved.                        *
 *                                                                            *
 * It may not be publicly disclosed, distributed, used, copied or modified    *
 * without prior written authorization by a representative of                 *
 * Communication Systems Group, Technische Universitt Berlin or the author.  *
 * Any modified version of this document needs to contain this header.        *
 ******************************************************************************/

/******************************************************************************
 * THERMS IF USAGE:                                                           *
 * PERSONAL, NON-COMMERCIAL or ACADEMIC USAGE:                                *
 *   You are free to use this software for whatever you like. If you use this *
 *   algorithm for a scientific publication, please cite the one of the       *
 *   following paper:                                                         *
 *																			  *
 * @INPROCEEDINGS{ICIPSenst2014,											  *
 * AUTHOR = {Tobias Senst and Thilo Borgmann and Ivo Keller and Thomas		  *
 * Sikora},																	  *
 * TITLE = {Cross based Robust Local Optical Flow},							  *
 * BOOKTITLE = {21th IEEE International Conference on Image Processing},	  *
 * YEAR = {2014},															  *
 * MONTH = okt,																  *
 * PAGES = {1967--1971},													  *
 * ADDRESS = {Paris, France},												  *
 * }																		  *
 *																			  *
 * @INPROCEEDINGS{ICIPSenst2013,											  *
 * AUTHOR = {Tobias Senst and Jonas Geistert and Ivo Keller and Thomas		  *
 * Sikora},																	  *
 * TITLE = {Robust Local Optical Flow Estimation using Bilinear Equations for *
 * Sparse Motion Estimation},												  *
 * BOOKTITLE = {20th IEEE International Conference on Image Processing},	  *
 * YEAR = {2013},															  *
 * MONTH = sep,																  *
 * PAGES = {2499--2503},													  *
 * ADDRESS = {Melbourne, Australia},										  *
 * DOI = {10.1109/ICIP.2013.6738515},										  *
 * }																		  *
 *                                                                            *
 * @ARTICLE{TCSVTSenst2012,                                                   *
 * AUTHOR = {Tobias Senst and Volker Eiselein and Thomas Sikora},             *
 * TITLE = {Robust Local Optical Flow for Feature Tracking},                  *
 * JOURNAL = {IEEE Transactions on Circuits and Systems for Video Technology},*
 * YEAR = {2012},                                                             *
 * MONTH = sep,                                                               *
 * PAGES = {1377--1387},                                                      *
 * VOLUME = {22},                                                             *
 * NUMBER = {9},                                                              *
 * DOI = {10.1109/TCSVT.2012.2202070}                                         *
 * }                                                                          *
 *                                                                            *
 * @INPROCEEDINGS{WACVSenst2011,                                              *
 * AUTHOR = {Tobias Senst and Volker Eiselein and Rubn Heras Evangelio and   *
 * Thomas Sikora},                                                            *
 * TITLE = {Robust Modified L2 Local Optical Flow Estimation and Feature      *
 * Tracking},                                                                 *
 * BOOKTITLE = {IEEE Workshop on Motion and Video Computing},                 *
 * YEAR = {2011},                                                             *
 * MONTH = jan,                                                               *
 * EDITOR = {Eric Mortensen},                                                 *
 * PAGES = {685--690},                                                        *
 * ADDRESS = {Kona, USA},                                                     *
 * DOI = {10.1109/WACV.2011.5711571},                                         *
 * }                                                                          *
 *                                                                            *
 * COMMERCIAL USAGE:                                                          *
 *   It is not allowed to use any content of this package for any commercial  *
 *   use or any advertisement for upcoming commercial products. If you want to*
 *   use any content for such a purpose please contact:                       *
 *   Prof. Dr.-Ing. Thomas Sikora <sikora@nue.tu-berlin.de>.                  *
 ******************************************************************************/

/******************************************************************************
 * WARRANTIES:                                                                *
 *                                                                            *
 * Software provided by Technische Universitt Berlin with this document is   *
 * provided "AS IS" and any express of implied warranties including, but      *
 * not limited to, the implied warranties of merchantability and fitness      *
 * for a particular purpose are disclaimed.                                   *
 * In no event shall the author or contributors be liable for any direct,     *
 * indirect, incidental, special, exemplary, or consequential damages         *
 * (including, but not limited to, procurement of substitute goods or         *
 * services, loss of use, data, or profits or business interruption) caused in*
 * any way out of the use of this software, even if advised of the possibility*
 * of such damage.                                                            *
 ******************************************************************************/
/** 
 * \file RLOFFactory.h
 * \brief RLOF factory.
 
 File contains the RLOF factory using the interface IFeatureTracker.
 * \version 1.2
 * \date 17.11.2014
 * \author Tobias Senst
 */

#pragma once
#ifndef _IRLOF_H_
#define _IRLOF_H_

#include <OpticalFlow/IFeatureTracker.h>

#pragma warning(disable: 4251) 
/*! Enumeration related to a specific PLK/RLOF derivative.
*/
enum RLOFType {  
				 CPU_RLOF = 0		/*!< Implementation running on the CPU using TBB and SSE, please <a href="index.html#Senst2012">Senst et al. 2012</a> or <a href="index.html#Senst2011">Senst et al. 2011</a>*/
				 
				,CPU_BERLOF			/*!< Implementation running on the CPU using TBB and SSE, please and refer to <a href="index.html#Senst2013">Senst et al. 2013</a>.*/
				,CPU_CROSS_RLOF		/*!< Implementation running on the CPU using TBB and SSE, please and refer to <a href="index.html#Senst2014">Senst et al. 2014</a>.*/
				,CPU_CROSS_BERLOF   /*!< Implementation running on the CPU using TBB and SSE, please and refer to <a href="index.html#Senst2014">Senst et al. 2014</a>.*/

				,CPU_BEPLK			/*!< Implementation running on the CPU using TBB and SSE, please and refer to <a href="index.html#Senst2013">Senst et al. 2013</a>.*/
				,CPU_CROSS_PLK		/*!< Implementation running on the CPU using TBB and SSE, please and refer to <a href="index.html#Senst2014">Senst et al. 2014</a>.*/
				,CPU_CROSS_BEPLK	/*!< Implementation running on the CPU using TBB and SSE, please and refer to <a href="index.html#Senst2014">Senst et al. 2014</a>.*/

				,CPU_PLK			/*!< OpenCV-PLK Implementation running on the CPU*/
			   };

/*! Convert RLOFType enumeration into a string
 *\param type enumeration specify a PLK/RLOF method
 *\return string refers to the enumeration
*/
static std::string RLOFTypeToString(RLOFType type)
{
	switch(type)
	{
		case CPU_PLK:			return std::string("CPU_PLK");		
		case CPU_BEPLK:			return std::string("CPU_BEPLK");	
		case CPU_CROSS_PLK:		return std::string("CPU_CROSS_PLK");	
		case CPU_CROSS_BEPLK:	return std::string("CPU_CROSS_BEPLK");	
		case CPU_RLOF:			return std::string("CPU_RLOF");		
		case CPU_BERLOF:		return std::string("CPU_BERLOF");	
		case CPU_CROSS_RLOF:	return std::string("CPU_CROSS_RLOF");	
		case CPU_CROSS_BERLOF:	return std::string("CPU_CROSS_BERLOF");	
	};
	return std::string("Not Defined");
}

/*! \class RLOFFactory
 *  \brief RLOF factory
 *
 *  To create an RLOF feature tracker you have to instantiate t using this class.
 *  The IFeatureTracker is used to run the RLOF feature tracker. The RLOFsample.cpp 
 *  shows how to do. Each object that
 *  was instantiated has to be delete outside to a void memory leaks.
*/
class DECL_SPEC RLOFFactory  
{
public:
 /*! Create an instance of a specified a PLK/RLOF implementation.
  *\param type RLOFType specifies the PLK/RLOF method 
   *\return Returns an instance of the specified PLK/RLOF implementation.
   */
	static IFeatureTracker *  createFTInstance(RLOFType type = CPU_BEPLK);
	/*! Create an instance of a specified a PLK/RLOF implementation from an given context.
	*\param context context specifies the  PLK/RLOF method
	*\return Returns an instance of the specified PLK/RLOF implementation.
	*/
	static IFeatureTracker *  createFTInstance(const FeatureTrackerContext * context);
};

#endif