/******************************************************************************
 * COPYRIGHT:                                                                 * 
 *                                                                            *
 * This file is the property of the author and Communication Systems Group,   *
 * Technische Universitt Berlin. All rights reserved.                        *
 *                                                                            *
 * It may not be publicly disclosed, distributed, used, copied or modified    *
 * without prior written authorization by a representative of                 *
 * Communication Systems Group, Technische Universitt Berlin or the author.  *
 * Any modified version of this document needs to contain this header.        *
 ******************************************************************************/

/******************************************************************************
 * THERMS IF USAGE:                                                           *
 * PERSONAL, NON-COMMERCIAL or ACADEMIC USAGE:                                *
 *   You are free to use this software for whatever you like. If you use this *
 *   algorithm for a scientific publication, please cite the one of the       *
 *   following paper:                                                         *
 *                                                                            *
 * @ARTICLE{Senst2012,                                                        *
 * AUTHOR = {Tobias Senst and Volker Eiselein and Thomas Sikora},             *
 * TITLE = {Robust Local Optical Flow for Feature Tracking},                  *
 * JOURNAL = {IEEE Transactions on Circuits and Systems for Video Technology},*
 * YEAR = {2012},                                                             *
 * MONTH = sep,                                                               *
 * PAGES = {1377--1387},                                                      *
 * VOLUME = {22},                                                             *
 * NUMBER = {9},                                                              *
 * DOI = {10.1109/TCSVT.2012.2202070}                                         *
 * }                                                                          *
 *                                                                            *
 * @INPROCEEDINGS{Senst2011,                                                  *
 * AUTHOR = {Tobias Senst and Volker Eiselein and Rubn Heras Evangelio and   *
 * Thomas Sikora},                                                            *
 * TITLE = {Robust Modified L2 Local Optical Flow Estimation and Feature      *
 * Tracking},                                                                 *
 * BOOKTITLE = {IEEE Workshop on Motion and Video Computing},                 *
 * YEAR = {2011},                                                             *
 * MONTH = jan,                                                               *
 * EDITOR = {Eric Mortensen},                                                 *
 * PAGES = {685--690},                                                        *
 * ADDRESS = {Kona, USA},                                                     *
 * DOI = {10.1109/WACV.2011.5711571},                                         *
 * }                                                                          *
 *                                                                            *
 * COMMERCIAL USAGE:                                                          *
 *   It is not allowed to use any content of this package for any commercial  *
 *   use or any advertisement for upcoming commercial products. If you want to*
 *   use any content for such a purpose please contact:                       *
 *   Prof. Dr.-Ing. Thomas Sikora <sikora@nue.tu-berlin.de>.                  *
 ******************************************************************************/

/******************************************************************************
 * WARRANTIES:                                                                *
 *                                                                            *
 * Software provided by Technische Universitt Berlin with this document is   *
 * provided "AS IS" and any express of implied warranties including, but      *
 * not limited to, the implied warranties of merchantability and fitness      *
 * for a particular purpose are disclaimed.                                   *
 * In no event shall the author or contributors be liable for any direct,     *
 * indirect, incidental, special, exemplary, or consequential damages         *
 * (including, but not limited to, procurement of substitute goods or         *
 * services, loss of use, data, or profits or business interruption) caused in*
 * any way out of the use of this software, even if advised of the possibility*
 * of such damage.                                                            *
 ******************************************************************************/
/** 
 * \file cvwrapper.hpp
 * \brief OpenCV wrapper.
 
 Provides a set of useful functions and classes to ensure the compatibility between different OpenCV version.
 * \date 04.05.2012
 * \author Tobias Senst
 */

#pragma once 
#ifndef _CVWRAPPER_HPP_
#define _CVWRAPPER_HPP_

#ifndef _NOT_USE_OPENCV_
#include <opencv2\core\core.hpp>
#endif

//! Wrapper cv::Point2f
/*! Wrapper class to substitute the cv::Point2f class. 
*/
class CRPoint
{
public:
	//! Default constructor
	CRPoint()
		: x(0)
		, y(0)
	{}
	/*! Constructor with initialisations.
	*\param _x Value of the x position
	*\param _y Value of the y position
	*/
	CRPoint(const float _x, const float _y)
		: x(_x)
		, y(_y)
	{}
	float x; /*!< x position:*/
	float y; /*!< y position:*/
};
#ifndef _NOT_USE_OPENCV_
/*! Wrapping function to convert std::vector<cv::Point2f> to std::vector<CRPoint>
 *\param lhs Destination vector containing the source values
 *\param rhs Source vector.
*/
static void crTocv(std::vector<cv::Point2f> & lhs, const std::vector<CRPoint> & rhs)
{
	lhs.resize(rhs.size());
	std::vector<CRPoint>::const_iterator i = rhs.begin();
	std::vector<cv::Point2f>::iterator	 n = lhs.begin();
	for(; i != rhs.end(); ++i, ++n)
	{
		n->x = i->x;
		n->y = i->y;
	}
}
/*! Wrapping function  to convert std::vector<CRPoint> to std::vector<cv::Point2f>
*\param lhs Destination vector containing the source values
 *\param rhs Source vector.
*/
static void cvTocr(std::vector<CRPoint> & lhs, const std::vector<cv::Point2f> & rhs)
{
	lhs.resize(rhs.size());
	std::vector<cv::Point2f>::const_iterator i = rhs.begin();
	std::vector<CRPoint>::iterator	 n = lhs.begin();
	for(; i != rhs.end(); ++i, ++n)
	{
		n->x = i->x;
		n->y = i->y;
	}
}
#endif


#endif