% /******************************************************************************
%  * COPYRIGHT:                                                                 * 
%  *                                                                            *
%  * This file is the property of the author and Communication Systems Group,   *
%  * Technische Universitt Berlin. All rights reserved.                        *
%  *                                                                            *
%  * It may not be publicly disclosed, distributed, used, copied or modified    *
%  * without prior written authorization by a representative of                 *
%  * Communication Systems Group, Technische Universitt Berlin or the author.  *
%  * Any modified version of this document needs to contain this header.        *
%  ******************************************************************************/
% 
% /******************************************************************************
%  * THERMS IF USAGE:                                                           *
%  * PERSONAL, NON-COMMERCIAL or ACADEMIC USAGE:                                *
%  *   You are free to use this software for whatever you like. If you use this *
%  *   algorithm for a scientific publication, please cite the one of the       *
%  *   following paper:                                                         *
%  *																			  *
%  * @INPROCEEDINGS{ICIPSenst2014,											  *
%  * AUTHOR = {Tobias Senst and Thilo Borgmann and Ivo Keller and Thomas		  *
%  * Sikora},																	  *
%  * TITLE = {Cross based Robust Local Optical Flow},							  *
%  * BOOKTITLE = {21th IEEE International Conference on Image Processing},	  *
%  * YEAR = {2014},															  *
%  * MONTH = okt,																  *
%  * PAGES = {1967--1971},													  *
%  * ADDRESS = {Paris, France},												  *
%  * }																		  *
%  *																			  *
%  * @INPROCEEDINGS{ICIPSenst2013,											  *
%  * AUTHOR = {Tobias Senst and Jonas Geistert and Ivo Keller and Thomas		  *
%  * Sikora},																	  *
%  * TITLE = {Robust Local Optical Flow Estimation using Bilinear Equations for *
%  * Sparse Motion Estimation},												  *
%  * BOOKTITLE = {20th IEEE International Conference on Image Processing},	  *
%  * YEAR = {2013},															  *
%  * MONTH = sep,																  *
%  * PAGES = {2499--2503},													  *
%  * ADDRESS = {Melbourne, Australia},										  *
%  * DOI = {10.1109/ICIP.2013.6738515},										  *
%  * }																		  *
%  *                                                                            *
%  * @ARTICLE{TCSVTSenst2012,                                                   *
%  * AUTHOR = {Tobias Senst and Volker Eiselein and Thomas Sikora},             *
%  * TITLE = {Robust Local Optical Flow for Feature Tracking},                  *
%  * JOURNAL = {IEEE Transactions on Circuits and Systems for Video Technology},*
%  * YEAR = {2012},                                                             *
%  * MONTH = sep,                                                               *
%  * PAGES = {1377--1387},                                                      *
%  * VOLUME = {22},                                                             *
%  * NUMBER = {9},                                                              *
%  * DOI = {10.1109/TCSVT.2012.2202070}                                         *
%  * }                                                                          *
%  *                                                                            *
%  * @INPROCEEDINGS{WACVSenst2011,                                              *
%  * AUTHOR = {Tobias Senst and Volker Eiselein and Rubn Heras Evangelio and   *
%  * Thomas Sikora},                                                            *
%  * TITLE = {Robust Modified L2 Local Optical Flow Estimation and Feature      *
%  * Tracking},                                                                 *
%  * BOOKTITLE = {IEEE Workshop on Motion and Video Computing},                 *
%  * YEAR = {2011},                                                             *
%  * MONTH = jan,                                                               *
%  * EDITOR = {Eric Mortensen},                                                 *
%  * PAGES = {685--690},                                                        *
%  * ADDRESS = {Kona, USA},                                                     *
%  * DOI = {10.1109/WACV.2011.5711571},                                         *
%  * }                                                                          *
%  *                                                                            *
%  * COMMERCIAL USAGE:                                                          *
%  *   It is not allowed to use any content of this package for any commercial  *
%  *   use or any advertisement for upcoming commercial products. If you want to*
%  *   use any content for such a purpose please contact:                       *
%  *   Prof. Dr.-Ing. Thomas Sikora <sikora@nue.tu-berlin.de>.                  *
%  ******************************************************************************/
% 
% /******************************************************************************
%  * WARRANTIES:                                                                *
%  *                                                                            *
%  * Software provided by Technische Universitt Berlin with this document is   *
%  * provided "AS IS" and any express of implied warranties including, but      *
%  * not limited to, the implied warranties of merchantability and fitness      *
%  * for a particular purpose are disclaimed.                                   *
%  * In no event shall the author or contributors be liable for any direct,     *
%  * indirect, incidental, special, exemplary, or consequential damages         *
%  * (including, but not limited to, procurement of substitute goods or         *
%  * services, loss of use, data, or profits or business interruption) caused in*
%  * any way out of the use of this software, even if advised of the possibility*
%  * of such damage.                                                            *
%  ******************************************************************************/

image1 = imread('img1.jpg');
image2 = imread('img50.jpg');

pointlist1 = [];

% perform an accuracy test of each PLK/RLOF implementation
% define parameter
parameter = [ struct('method', 'PLK', 'options', 'PerformTest')...
              struct('method', 'RLOF', 'options', 'PerformTest')...
              struct('method', 'BEPLK', 'options', 'PerformTest')...
              struct('method', 'BERLOF', 'options', 'PerformTest')...
              struct('method', 'CB_BEPLK', 'options', 'PerformTest')...
              struct('method', 'CB_BERLOF', 'options', 'PerformTest')...
              ];

% demo estimates and validates the motion vector for each PLK/RLOF derivate
idx = 1;
for n = 1 : length(parameter)
    pointlist2 = mex_RLOF(image1, image2, pointlist1, parameter(n));
end

% demo estimates and displays motion vectors for a given grid of samples 
gridSize = 10;
noRows = size(image1,1);
noCols = size(image1,2);
% initialize pointlist with features to track
[Y,X] = meshgrid(0:gridSize:noRows-1, 0:gridSize:noCols-1);
pointlist1 = [reshape(X, size(X,1) * size(X,2), 1)'; reshape(Y, size(Y,1) * size(Y,2), 1)' ];
%track features
parameter = struct('method', 'PLK', 'options', 'PrintParameter');
pointlist2 = mex_RLOF(image1, image2, pointlist1, parameter);
% draw result
imagesc(image1);
for i = 1:size(pointlist2,2)
    line([pointlist1(1,i) pointlist2(1,i)], [pointlist1(2,i) pointlist2(2,i)]);
end